import requests
import time, sys
import json, os
import logging
import shutil
import configparser
from os.path import join, dirname, abspath, normpath, basename
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
from cshmac.requests import HmacAuth

LOG_FILE_PATH = '/var/log/cyops/install/demoscript.log'

logging.basicConfig(filename=LOG_FILE_PATH, level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(module)s %(funcName)s(): %(message)s')

logger = logging.getLogger(__name__)

APPLIANCE_PUBLIC_KEY = ''
APPLIANCE_PRIVATE_KEY = ''
WORKFLOW_COLLECTIONS = '/api/3/workflow_collections'
STAGING_MODEL_MEATDATAS = '/api/3/staging_model_metadatas'
PUBLISH = '/api/publish'
SYSTEM_VIEW_TEMPLATES = '/api/3/system_view_templates'


def make_request(url, method, body=None):
    try:
        bodyless_methods = ['head', 'get']
        if method.lower() in bodyless_methods:
            body = None
        auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, json.dumps(body))
        response = requests.request(method, url, auth=auth, json=body, verify=False)
        if response.status_code == 409:
            logger.info('The entity already exists')
        elif response.status_code == 200 or response.status_code == 201 or response.status_code == 204:
            return response.json()
        elif response.status_code == 503:
            return response.json()
    except Exception as Err:
        logger.exception('Exception ocurred:'.format(str(Err)))


def post_playbook(playbook_url, body):
    try:
        response = make_request(playbook_url, 'POST', body=body)
        if response:
            logger.info("Playbook import response: {}".format(response))
    except requests.exceptions.Timeout as Err:
        logger.exception('Timeout exception: {}'.format(str(Err)))
    except requests.exceptions.RequestException as Err:
        logger.exception(str(Err))
        return


def trigger_playbook(main_url, payload=None):
    response = make_request(main_url, 'POST', body=APPLIANCE_PUBLIC_KEY)
    logger.info('Response for trigger playbook: {}'.format(response))


def get_playbook_data(ip_url, contents):
    playbook_url = ip_url + WORKFLOW_COLLECTIONS
    if 'data' in contents:
        data = contents["data"]
    else:
        data = contents["hydra:member"]
    for playbook in data:
        post_playbook(playbook_url, playbook)


def post_modules_in_staging_model_metadata(modules, main_url):
    api_url = main_url + STAGING_MODEL_MEATDATAS
    for module in modules:
        response = make_request(api_url, 'POST', body=module)
        if response:
            logger.info("Module created")
        else:
            logger.info("Module already exist")


def publish_module(iri):
    api_call = iri + PUBLISH
    response = make_request(api_call, 'PUT', body={})
    logger.info("Publish started")
    while True:
        ret = make_request(iri + '/api/publish/error', 'GET')
        if 'status' in ret and ret['status'] == 'Success':
            logger.info("Publish successful")
            return True
        elif 'status' in ret and ret['status'] == 'Failure':
            logger.info("Publish Failed")
            return False
        else:
            time.sleep(10)


def post_picklists(picklists, main_url):
    for element in picklists:
        name = element['name']
        if ' ' in name:
            name = name.replace(" ", "+")
        if 'uuid' in element:
            id = element['uuid']
            picklist_names_url = main_url + '/api/3/picklist_names?name=' + name
            picklist_names = make_request(picklist_names_url, 'GET')
            logger.info('picklist_names: {}'.format(picklist_names))
            if 'hydra:totalItems' in picklist_names and picklist_names['hydra:totalItems'] == 0:
                logger.info('Adding new picklist')
                element_list = []
                element_list.append(element)
                url = main_url + '/api/3/'
                data = make_request(url, 'POST', body=element_list)
                logger.info('data: {}'.format(picklist_names))
                logger.info('Picklist found')
            else:
                picklist_url = main_url + '/api/3/picklist_names/' + id + '?$relationships=true'
                result = make_request(picklist_url, 'GET')
                logger.info('Result: {}'.format(result))
                for j in element['picklists']:
                    picklist = j['itemValue']
                    for i in result['picklists']:
                        if picklist != i['itemValue']:
                            result_ret = make_request(main_url + '/api/3/picklist_names/' + id, 'PUT', body=j)
                            if result_ret:
                                logger.info('Adding new value to picklist')
                        else:
                            logger.info('Picklist not found')
        else:
            logger.info('UUID is missing')


def get_modelmetadata_svt_from_file(filename, main_url):
    with open(filename) as fp:
        data = fp.read()
        contents = json.loads(data)
        modules = contents["modules"]
        viewTemplates = contents["viewTemplates"]
        picklists = contents["picklistNames"]
        post_picklists(picklists, main_url)
        post_modules_in_staging_model_metadata(modules, main_url)
        res = publish_module(main_url)
        if res:
            for key, val in viewTemplates.items():
                for element in val:
                    # Add in SVT
                    response = make_request(main_url + SYSTEM_VIEW_TEMPLATES, 'POST', body=element)
                    return True
        else:
            return False


def read_files(key_path, appliance_path):
    global APPLIANCE_PUBLIC_KEY
    global APPLIANCE_PRIVATE_KEY
    try:
        with open(join(key_path, 'APPLIANCE_PRIVATE_KEY'), 'r') as private:
            APPLIANCE_PRIVATE_KEY = private.read()
        with open(join(key_path, 'APPLIANCE_PUBLIC_KEY'), 'r') as public:
            APPLIANCE_PUBLIC_KEY = public.read()
    except FileNotFoundError:
      src = appliance_path
      shutil.copy(join(src, 'APPLIANCE_PRIVATE_KEY'), key_path)
      shutil.copy(join(src, 'APPLIANCE_PUBLIC_KEY'), key_path)
      read_files(key_path, appliance_path)


def get_appliance_keys(config):
    appliance_path = config['DEFAULT']['APPLIANCE_PATH']
    app_path = os.path.dirname(os.path.abspath(__file__))
    key_path = join(os.path.dirname(os.path.abspath(__file__)), 'keys')
    read_files(key_path, appliance_path)


def main():
    config = configparser.ConfigParser()
    conf_path = join(dirname(abspath(__file__)), 'setup.conf')
    config.read(conf_path)
    main_url = 'https://' + config['DEFAULT']['MAIN_URL']
    get_appliance_keys(config)

    data_path = join(dirname(os.path.dirname(os.path.abspath(__file__))), 'data')
    for item in os.listdir(data_path):
        if item.startswith('Config - 01 - MMD and SVT'):
            if not get_modelmetadata_svt_from_file(data_path + '/' + item, main_url):
                break
        if item.startswith('Playbooks'):
            with open(data_path + '/' + item, 'r', encoding='utf-8') as fp:
                contents = json.loads(fp.read())
            get_playbook_data(main_url, contents)

    endpoint = 'demo_records'
    url = main_url + '/api/triggers/1/' + endpoint
    trigger_playbook(url)


if __name__ == "__main__":
    main()
